/*
Autom.h - Library for creating a PLC 

by Cordier Yves Nov 2016 Version du 30/11/2016


This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/* Cette bibliothèque permet d'implementer un automate programable
à structure grafcet respectant les 5 règles du grafcet :
Le Grafcet fonctionne en suivant 5 règles d'évolution :

Règle N°1 :

L'initialisation précise l'étape ou les étapes actives au début du fonctionnement. 
On la repère en doublant les côtés des symboles correspondants. 
Il peut y avoir plusieurs étapes initiales dans un grafcet.

Remarque : Les étapes initiales sont activées inconditionnellement en début de cycle.

Règle N°2 :

	Une transition est soit validée, soit non validée. Elle est validée lorsque 
	toutes les étapes immédiatement précédentes sont actives.
	Elle ne peut être franchie que :
	Lorsqu'elle est validée, et que la réceptivité associée à la transition est vraie.

Règle N°3 :

	Le franchissement d'une transition entraîne l'activation simultanée de 
	toutes les étapes immédiatement suivantes et la désactivation de 
	toutes les étapes immédiatement précédentes.

	
Règle N°4 :

	Plusieurs transitions simultanément franchissables sont simultanément franchies. 
	
Règle N°5 :

	Si au cours du fonctionnement, une même étape doit être activée et désactivée 
	simultanément, elle reste active. 

Forcage/figeage

Une opération de figeage de graphes conduit à une non évolution du graph (les transitions sont déclarées non 
franchissables

Une opération de forcage de graph conduit à un pré-positionnement du système de graphes (partiel ou total)

Un graph figé peut évoluer par forçage.
	
Pour pouvoir suivre ces règles et limiter le travail de l'utilisateur,
plusieurs types d'objet sont implémentés. 
L'utilisateur ne se souciera que de très peu de types :

Le projet : class project
***********

Ceet classe est en fait la colonne vertébrale de tout le système. 
Elle aglomère l'ensemble des entités du projet automate et intègre
le fonctionnement respectant les règles.
ex project P1=project();

Comme tout automate ndustriel, vous écrivez vos trois parties
void preliminaire_appli()
{ 
 on y traite les fronts et autre bizareries
}

void calculEvolution_appli()
{
 on y décrit la méthode d'évolution des graphes voir exemples
}

void posterieur_appli()
{
 on affecte les sorties, armement tempos ...
}

L'implentation de ces fonctions dans notre application se fait dans le setup par :

 on affecte les fonctions
appli.setPreliminaire(preliminaire_appli);
appli.setCalculEvolution(calculEvolution_appli);
appli.setPosterieur(posterieur_appli);


qui donne lieu dans la fonction loop à :
void loop()
{
initloop(); // ligne obligatoire pour pouvoir gérer les tempos, temps de cycle

P1.fonctionnement(); // et c'est tout (ou presque)
}

Dans ce projet, on va installer les divers éléments par la fonctio add(

Les bascules : class bascule
*************

ex bascule B1=bascule(bascule1Prio,"bas1"); 
=> création d'une bascule à mise à 1 prioritaire, nommée bas1
bascule B2=bascule(bascule0Prio); 
=> création d'une bascule à mise à 0 prioritaire  sans nom => indebogable individuellement

dans le setup, on trouvera P1.add(B1); P1.add(B2); ...

Les tempo : class tempo
***********

il existe trois types de tempo
tempoTon : temporisation à l'enclenchement
tempoTof : temporisation au declenchement
tempoTpulse : une impulsion calibrée

ex tempo T1=tempo(tempoTpulse,2500); => Création d'une tempo de type pulse de 2500 ms

vous avez compris, on installe cette tempo dans le projet au niveau du setup par P1.add(T1);
pour l'utilisation, se referer à l'exemple tempo

Les compteurs : class compt
***********

ce sont des compteurs de nombre entiers entre -2147483648 et 2147483647
ils fonctionnent sur front montant de l'entrée de commande
Attention le depassement n'est pas géré => une incrementation à partir de 2147483647 --> -2147483648
								de même une decrementation à partir de -2147483648 --> 2147483647
mais on a le temps !!


ex compt C1=compt(100,"C pieces"); => Création d'un compteur à preselection de 100 nommé C PIECES

de même on installe ce compteur dans le projet au niveau du setup par P1.add(C1);
pour l'utilisation, se referer à l'exemple compt

Les PID : class pid
************

Ces structures permettent d'implementer un contrôle PID sur une sortie analogique

ex pid Pi1=pid(20,1,0,"pid1"); => Création d'un PID avec coef propo de 20 dérivé de 1 et intégral de 0 nommé pid1

de même on installe ce pide dans le projet au niveau du setup par P1.add(Pi1);
pour l'utilisation, se référer à l'exemple.


Les graphes : class graph
************

On ne creee pas les étapes indépendantes mais tout un graphe
ex graph G1=graph(6,1,"graph1"); création d'un graph de 6 étapes (de 0 à 5) 
avec une étape initiale N°1, le graphe possède le nom de baptème graph1 (util pour debogage)
si par hasard, notre graph possédait 
- aucunes étapes initiales G2=graph(6,"graph2");
-  deux étapes initiales (0 et 2)
G3=graph(6,0,"graph3"); puis au niveau du setup, on déclare G3.setInit(2);
On installe tout le monde dans le projet par
P1.add(G1); P1.add(G2); ...

pour l'utilisation, se référer à l'exemple grafcet

Les entrées sorties séries : class serialInOut
**************************

ces modules permettent d'envoyer et ou recevoir des informations via une sortie série
(attention dans le cas du Uno, un seul serialisateur matériel existe. 
Cette "cerise sur le gateau" sera donc à employer avec un module plus puissant.
ex serialInOut ser=serialInOut(&Serial2,4800,"dialogue"); 
vous avez compris, on utilise le sérialisateur matériel serial2 à 4800 bauds, nommé dialogue

comme pour les autres modules, on l'installe par 
P1.add(ser)

L'automate ainsi créé possède un mode de debogage si celui-ci est raccordé à une console
la mise en place de la console  se fait par la ligne suivante :

exemple de Mise en place du mode de debogage sur la sortie serie Serial (pour un uno)
on peut utiliser serial2, serial3 ... pur des modules équipés de processeurs
suppérieurs au uno de base

appli.debugMode(&Serial,9600); // mise en place du mode de debogage

les commandes de la console sont les suivantes : 

// Les commandes de la console :
// *****************************

#R ou #r  : mode run
#S ou #s  : mode stop
#I ou #i  : réinitialisation à l'état d'origine
#D ou #d  : mode debug
#N ou #n  : mode non debug
#L ou #l  : affiche le dernier temps de cycle (celui sans mode debug consommateur de temps)


pour le mode debug, on peut interroger l'état par morceau

?   : tout est montré sauf le temps de cycle
?*  : tout est montré sauf le temps de cycle
?*g : tous les (g)raphes
?*t : toutes les (t)empos
?*c : tous les (c)ompteurs
?*p : tous les PID
?*b : toutes les (b)ascules
?*s : les entrées sorties (s)erie
?*l : temps de cyc(l)e   

?<nom> : l'objet de nom <nom> 

lors d'un envoi d'un ordre de commande, le système répond en renvoyant l'ordre compris
Attention le debogage consomme beaucoup de temps. Les temps de cycle remontés par
la sucession de commande 
#D
?*l
ne sont guère fiables
par contre
la sucession de commande
#N
?*l
renvoi le dernier temps de cycle avant traitement
du debogage. Ce dernier est relativement fiable.

*/

// =============================================================
// La bibliothque d'automatisme avec ses éléments de base
// =============================================================

#ifndef Autom_h
#define Autom_h


// Les constantes internes au module utilisées dans les class bascule et etape
#define etat_actif 0x01
#define etat_fige 0x02
#define etat_mise_a_1 0x04
#define etat_mise_a_0 0x08
#define etat_forcage_0 0x10
#define etat_forcage_1 0x20
#define etat_mise_a_1_prio 0x40
#define etat_initial 0x80
#define etat_fmt 0x01
#define etat_fdcd 0x02
#define etat_bits_config etat_initial+etat_mise_a_1_prio

// les constantes internes utilisé par la classe tempo
#define etat_ton 0x10
#define etat_tof 0x20
#define etat_tpulse 0x40
#define tempoTon etat_ton
#define tempoTof etat_tof
#define tempoTpulse etat_tpulse

// Les constantes internes utilisées dans la classe compteur
#define etat_sup 0x02
#define etat_inf 0x04

// les constantes utilse par la classe bascule
#define bascule0Prio  0x00
#define bascule1Prio  etat_mise_a_1_prio

#define maxUnsignedLong  4294967295

// constante utilisée dans la class serialInOut
#define maxBuffer 128 //la taille du buffer serialInOut

// constantes utilisées dans la class pid pour definir les coef
#define paralCoef 0x00
#define paralTime 0x01
#define mixteCoef 0x10
#define mixteTime 0x11

//=================================================
// variables globales utiles et fonctions associées
//=================================================
unsigned long tempsCourantBibliAutomMillis = millis(); 
unsigned long tempsCourantBibliAutomMicros = micros();
void initLoop();

//===============================================================
// Petites fonctions utiles
//===============================================================
// Fonction utilisée pour le calcul des delai
// résoud le problème de "l'enroulement" des nombres

unsigned long Delai_entre(unsigned long T1, unsigned long T2);

// fonction qui ne fait rien pour initialiser 
void doNothing();

// Fonctions et variables utilisées pour le debogage
HardwareSerial *SerialDebug = NULL; // La sortie série pour la fonction debug
boolean SerialDebugInit = false;

// *******************************
// type interne à la bibliotheque
union numType 
{
	byte b;
	int i;
	long l;
	float f;
	double d;
	byte t[sizeof(double)];
};
//===============================================================
// Les objets de la bibliotheque
//===============================================================
// L'entrée-sortie série 


class serialInOut
{
private:
	HardwareSerial *SerialConnected = NULL; // La sortie série pour la fonction debug
	boolean SerialConnectedInit = false;
	byte bufferIn[maxBuffer];
	byte bufferOut[maxBuffer];
	int endBufferIn=0;
	int endBufferOut=0;
	int nbByteSend=0;
	
public:
	String nom="";
	unsigned int speed=9600;
	// Le constructeur amélioré amélioré
	serialInOut(HardwareSerial *S=&Serial , unsigned int s=9600, String n="");
	// baptise la sortie serie
	void setName(String n);
	// renvoi le nb de byte dans le buffer d'entrée -1 si aucuns
	int available();
	// prend le premier byte mis dans le buffer d'entrée (FIFO) -1 si aucuns 
	int get();
	// renvoi un entier (2 octets du buffer) 0 si impossible
	int getInt();
	// renvoi un entier long (4 octets du buffer) 0 si impossible
	long getLong();
	// renvoi un flotant (4 octets) 0 si impossible;
	float getFloat();	
	// renvoi un flotant double (8 octets) 0 si impossible;
	double getDouble();	
	// prend une chaine de caractère du buffer d'entrée
	void getString(String &s);
	// met un byte dans le buffer de sortie (FIFO) et pousse
	void put(byte b);
	// attention ne gère pas les débordements
	// mets un entier (2 octets) dans le buffer de sortie
	void put(int n);
	// mets un entier de type long(4 octets) dans le buffer de sortie
	void put(long n);
	// mets un flotant(4 octets) dans le buffer de sortie
	void put(float f);
	// mets un flotant double(8 octets) dans le buffer de sortie
	void put(double d);
	// mets une chaine de caractère azt dans le buffer de sortie
	void put(String s);
	// vide les buffers
	void reInit();
	// va remplir le bufferin si byte dispo
	void preliminaire();
	// va ecrire sur la sortie serie le bufferout préparé
	void posterieur();
	// mode deboggage renvoi les infos vitesse position des buffers ...
	void debug();
private:	
	// decale le buffer de sortie de n crans vers la droite, oublie les bytes qui débordent
	void decaleDroite(int n=1);
	// recoit n caractères du buffer si possible
	boolean recoit(numType &d,byte n);
	// emmet n caractères 
	void emmet(numType &d, byte n);
};


// La bascule qui est l'element de base de l'automatisme
//******************************************************

// La classe bascule
class bascule
{ // public => visible de tous le monde
	//protected => visible des descendants
protected:
	// Le byte etat de la bascule contient toutes les informations
	byte etat;
	// Evolution Mise à un prioritaire
	void evolutionUnPrioritaire();
	// Evolution Mise à zero prioritaire
	void evolutionZeroPrioritaire();
	// précise le type de bascule
	void typeBascule(byte act);
public:
	// Le nom de la bascule
	String nom="";
	// constructer en précisant le type et le nom
	bascule(byte act=bascule1Prio,String n="");
	// retour de l'état de la bascule
	boolean isActif();
	// activation ou desactivation de la bascule
	void setActif(boolean act);
	// forcage de la bascule à 0 ou 1
	void setForcage(boolean act);
	// calcul de l'évolution après pré-positionnement
	void evolution();
	// la réinitialisation
	void reInit() {etat=etat & etat_bits_config;}
	// pour le debogage
	void debug();
	// donner un nom 
	void setName(String n);
	void set() {setActif(true);};
	void reset() {setActif(false);};
};

// l'étape Grafcet qui est une bascule améliorée
//******************************************************

// L'étape respecte les 5 règles du grafcet

class etape : public bascule // sous classe de la bascule
{
	protected : // visible des descendants
	byte etat_front;
public:
	// Constructeur evolué (pour definir les étapes initiales)
	etape(boolean initial=false);
	// ramène les étapes à leurs valeurs initiales
	void reInit();
	// surcharge de la methode evolution pour traiter les fronts
	void evolution();
	// gestion des fronts
	boolean fmt();
	boolean fdcd();
	boolean rising() {return fmt();};
	boolean falling() {return fdcd();};
	// defini le type de l'etape (initial : true ; normale : false)
	void typeEtape(boolean t);
	// fige l'étape sur le tour de calcul (action monostable)
	void setFigeage();
	
};

// les graphes
//******************************************************

class graph
{ protected :
	etape *et; // Les étapes du graphe et est un pointeur vers un tableau d'étapes
	unsigned int nb; // Nombre d'étapes
public:
	String nom = "";
	// Le constructeur super amélioré (un graphe à n étapes et une étape initiale N°i) et un nom s
	graph(int n=0,int i=0,String s="");
	// Creation proprement dite du graphe
	void creation(int n);
	// Preparation du graphe (création étapes initiales)
	void init( int n); // n numero etape initiale n >=0 n<nb
	// Ramener le graph à son état initial et le demarre (initialise les étapes initiales);
	void reInit();
	// calcul des évolution du graphe
	void evolution();
	// renvoi le pointeur sur l'etape numero n
	etape* contenu(unsigned int n);
	// fige le graphe pour evolution (un graphe figé peut évoluer par forcage)
	void fige();
	// pour le mode debugging necessite d'avoir initialisé au préalable la sortie
	// serialdebug par la fonction debugmode(hardwareSerial *S)
	void debug(String n);
	// affecte un nom au graph
	void setName(String n);
	// renvoi le nombre d'étapes
	int getNb();
	// desactive le graph
	void unActiv();
	// force à 1 une etape donnée
	void forceTo1(int i);
	
};

// Les temporiseurs à base variable (par defaut milliseconde)
//***********************************************************
// Les temporiseurs à base variable (par defaut milliseconde)
//***********************************************************

class tempo
{
protected:
	byte etat;
	unsigned long tinit;
	unsigned long tempsEcoule = 0;
	unsigned long delaisMs;
	boolean actived=false; // indique si la tempo est activée
	boolean activeSto=false; 
	boolean activationAsked = false; // indique si on a demandé l'activation
	boolean reinitAsked=false; // indique si on a demandé une reinitialisation (prioritaire)
	
	void evolutionTon(); // evolution pour tempo de type ton (tempo à l'enclenchement)
	
	void evolutionTof();// evolution pour tempo de type tof (tempo au declenchement) 
	
	void evolutionTpulse(); // evolution pour tempo de type tpulse (impulsion calibrée)

	void resetIndic(); // remet les indicateurs au repos
	
public:
	String nom = "";
	// Le constructeur par defaut (base ms type ton : temporisateur à l'enclenchement 
	tempo();
	// Le constructeur en choisissant le type , le temps et le nom
	tempo(byte t=tempoTon,unsigned long tpulse=0,String s=""); 
	// est elle terminée ? renvoi l'état de la sortie
	boolean out();
	// force la désactivation
	void reInit();
	// active la tempo sur front montant de l'entree
	void active();
	// evolution de la tempo 
	void evolution();
	// le nom
	void setName(String n);
	// debugging
	void debug();
	
};

// Les compteurs/decompteurs à préselection
//***********************************************************
// Les actions sur compteurs sont par défaut sur front montant
//***********************************************************

class compt
{
protected:
	byte etat;
	long valeur;
	long presel= 0;
	boolean activedinc=false; // indique si la tempo est activée en incrementation
	boolean activeddec=false; // indique si la tempo est activée en decrementation
	boolean activedreinit=false; // reinitialise le compteur à la preselection
	boolean incrementAsked = false; // indique si on a demandé l'incrementation
	boolean decrementAsked = false; // indique si on a demandé la decrementation
	boolean modifPreselAsked =false;  // indique que l'on a demandé la modification de la preselection
	boolean reinitAsked=false; // indique si on demande une reinitialisation à presel (prioritaire)
	boolean resetAsked=false; // indique une demande de rest (mise à zero)
	long newPresel=0;
	void resetIndic();
	
public:
	String nom = "";
	// Le constructeur par defaut (presel=0)
	compt();
	// Le constructeur en choisissant la preselection et le nom
	compt(long pre=0,String s=""); 
	// est on à la preselection ?
	boolean isActif();
	boolean isEqual();
	// est on à la preselection ?
	boolean isUpper();
	// est on à la preselection ?
	boolean isLower();
	// la valeur courante est elle zero ?
	boolean isZero();
	// renvoi la valeur du compteur (sous forme long)
	long val();
	// modifie la preselection
	void setPresel(long p);
	// force la réinitialisation à la preselection
	void preSet();
	// reset la valeur du compteur à 0
	void reSet();
	void reInit(); // créé par compatibilité avec les autres entités
	// compte sur front montant de l'entree
	void comptUp();
	// decompte sur front montant de l'entree
	void comptDown();
	// evolution de la tempo 
	void evolution();
	// le nom
	void setName(String n);
	// debugging
	void debug();
};

// Structure pour "aglomerer" les pid
// **************************************
class pid
{
protected:
	double epsilonSto=0;
	double retour=0;
	double sumValue=0;
	double sortie=0;
	double maxi=255;
	double mini=0;
	unsigned long dernierTraitement;	
    unsigned long delaiScrutation=100000; // 100 ms	(en microsecondes)
	double Kp=0;
	double Kd=0;
	double Ki=0;
	unsigned long Delta = 200; // ms;
	boolean etat=true;
	byte mode=paralCoef;
	
public:
	String nom = "";
	// Le constructeur en choisissant la preselection et le nom
	pid( double Kp=1, double Kd=0, double Ki=0, String s=""); 
	// créé par compatibilité avec les autres entités
	void reInit(); 
	// initialise la valeur courante de retour
	void setFeedBack(double v);
	// evolution du pid 
	void evolution();
	// le nom
	void setName(String n);
	// debugging
	void debug();	
	// set new deltat;
	void setDeltaT(unsigned long d);
	//calibre la sortie dans une plage donnée. 0-255 par defaut (pilotage d'une sortie anna 8 bits)
	void setOutputLimits(double max, double min); 
	// arrete ou met en marche le pid
	void on(); // redemarre le pid à partir de la dernière valeur de sortie
	void off(); // arrete le pid et fige la valeur de sortie
	double out(); // valeur de sortie
	void setOut(double s);
	// mets de nouveaux coef
	void setCoefParal(double Kpo, double Kde, double Kin);
	void setCoeftimeParal(double Kpo, double Tde, double Tin);
	void setCoefmixte(double Kpo, double Kde, double Kin);
	void setCoeftimemixte(double Kpo, double Tde, double Tin);
	boolean state();

};

// Structure pour "aglomerer" les pid
// **************************************

class pilePid
{
	protected :
	pilePid *next; // L'objet suivant
	pid *contents; // Vers l'objet pointé
public:	
	pilePid(pid *g=NULL);
	void evolution();
	void debug(String n);
	void add(pid *g);
	void reInit();	
};

// Structure pour "aglomerer" les graphes
// **************************************
class pileGraph
{ 
	protected :
	pileGraph *next; // L'objet suivant
	graph *contents; // Vers l'objet pointé
public:	
	pileGraph(graph *g=NULL);
	void evolution();
	void debug(String n);
	void add(graph *g);
	void reInit();
};

// Structure pour "aglomerer" les tempos
// **************************************
class pileTempo
{ 
protected:
	pileTempo *next; // L'objet suivant
	tempo *contents; // Vers l'objet pointé
	public :
	pileTempo(tempo *t=NULL);
	void debug(String n);
	void add(tempo *t);
	void reInit();
	void evolution();
};
// Structure pour "aglomerer" les compteurs
// ****************************************
class pileCompt
{ 
protected:
	pileCompt *next; // L'objet suivant
	compt *contents; // Vers l'objet pointé
	public :
	pileCompt(compt *c=NULL);
	void debug(String n);
	void add(compt *t);
	void reInit();
	void evolution();
};
// Structure pour "aglomerer" les bascules
// **************************************
class pileBascule
{ 
protected:
	pileBascule *next; // L'objet suivant
	bascule *contents; // Vers l'objet pointé
public:
	pileBascule(bascule *b=NULL);
	void debug(String n);
	void add(bascule *b);
	void reInit();
	void evolution();
};

// Structure pour "aglomerer" les serialInOut
// ******************************************
class pileSerialInOut
{ 
protected:
	pileSerialInOut *next; // L'objet suivant
	serialInOut *contents; // Vers l'objet pointé
public:
	pileSerialInOut(serialInOut *s=NULL);
	void preliminaire();
	void posterieur();
	void debug(String n);
	void add(serialInOut *s);
	void reInit();
};
// Structure pour tout "aglomerer"
// *******************************
class project
{
	protected :
	// Les fonctions adaptées à chaque appli
	void (*action1)(); // la sucession des actions en fait
	void (*action2)(); // les *action<n> () sont des pointeurs vers des fonctions sans parametre
	void (*action4)(); // nous les remplacerons par la suite par les fonctions qui nous intéressent
	
	// Traitement des commandes venant de la console
	void executeCommande(); 
	// Reinitialisation du projet
	void reInit();
	// evolution des entités apres action1, action2 et avant action4
	void evolution();
	// fonctions préliminaires non négociables
	void preliminaire();
	// fonctions postérieures non négociables
	void posterieur();
	void debugTempsCycle(); // affiche le temps de cycle sur la console
	boolean mode=true; // Le mode de fonctionnement true : Run ; false : stop 
	boolean modeDebug = false; // le mode de debogage piloté par les commandes #D ou #N
	String ligneCommande="";
	String ligneDebug="";
	unsigned long tempsAvant;
	unsigned long tempsCycle =0;
	// Fonctions et variables utilisées pour le debogage
	HardwareSerial *SerialDebugProject = NULL;
	boolean SerialDebugInitProject = false;
	HardwareSerial *SerialDebugSto = NULL;
	boolean SerialDebugInitSto = false;;
	


public:
	String nom="";
	project(String n=""); // Le constructeur lors de l'appel le nom est optionnel
	pileBascule *pB; // les objets de type bascule
	pileTempo *pT; // les objets de type tempo
	pileCompt *pC; // les objets de type compteur
	pileGraph *pG; // les objets de type graph
	pileSerialInOut *pS; // les objets de type serialInOut
	pilePid *pPi; // les objets de type pid;
	void debug(String n); // mode de debogage
	void add(tempo *T); // ajout d'un objet de type tempo
	void add(compt *C); // ajout d'un objet de type compt
	void add(bascule *B); // ajout d'un objet de type bascule
	void add(graph *G); // ajout d'un objet de type graph
	void add(serialInOut *S); // ajout d'un objet de type serialInOut
	void add(pid *P);// ajoute un pid au projet  
	// mise en place des actions crées par l'utilisateur 
	void setPreliminaire(void p()) {action1=p;}; // en paramètre, le pointeur vers la fonction que nous désirons
	void setCalculEvolution(void p()){action2=p;}; // mettre en place dans action<n>
	void setPosterieur(void p()){action4=p;};
	// Le coeur de fonctionnement qui lance les actions<n>
	void fonctionnement();
	long getCycle(); // retourne le dernier temps de cycle en microsecondes
	// initialise le mode debogage
	void debugMode(HardwareSerial *S=&Serial , unsigned int speed=9600); // par defaut on est sur Serial et 9600 bauds
}; 
#endif

/*
Autom.cpp - Library for creating a PLC 

by Cordier Yves Nov 2016 Version du 141/11/2016


This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//******************************************************************************
//* Includes
//******************************************************************************

// les entêtes
#if ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

// #include "Autom.h"

// Partie implementation des fonctions

// Fonction de début de boucle loop, permet d'utiliser correctement les temporisateurs et pid
void initLoop()
{
	tempsCourantBibliAutomMillis = millis();
	tempsCourantBibliAutomMicros=micros();
}
// Fonction utilisée pour le calcul des delai
// résoud le problème de "l'enroulement" des nombres

unsigned long Delai_entre(unsigned long T1, unsigned long T2)
{
	if (T2 >= T1)
	{
		return (T2 - T1);
	}
	else
	{
		return (T2 + maxUnsignedLong - T1 + T2);
	}
}

// fonction qui ne fait rien pour initialiser 
void doNothing()
{}


// Les méthodes de l'objet project
// *******************************

// Le constructeur pavec un nom
project::project(String n)
{
	pB= new pileBascule(); // la liste chainée des bascules
	pT= new pileTempo(); // la liste chainée des tempos
	pG= new pileGraph(); // la liste chainée des graphes
	pS= new pileSerialInOut();// la liste chainée des entrée sorties series
	pC= new pileCompt(); // la liste chainée des compteurs
	pPi= new pilePid(); // la liste chainée des Pid
	action1=doNothing; // par defaut et par securité,
	action2=doNothing; // on mets en place la fonction doNothing()
	action4=doNothing; // qui comme son nom l'indique ne fait rien
	ligneCommande=""; // On ne commande rien
	mode=true; // marche par defaut
	modeDebug = false; // pas de mode debug par defaut 
	tempsAvant=micros();
	n.toUpperCase();
	nom=n;
}


// pour le deboggage 
/* 
* ou rien signifie tout
*b toutes les bascules
*t toutes les tempos
*g tous les graphes
<autre> l'objet de nom <autre> ex "toto"
*/
void project::debug(String n)
{ 
	if (SerialDebugInit)
	{
		if (n.length()>0) 
		{
			if (n[0]=='*')
			{if (n.length()>1)
				{
					if ((n[1]=='*')||(n[1]=='B')) {pB->debug("*");}; // debogage bascules
					if ((n[1]=='*')||(n[1]=='T')) {pT->debug("*");}; // debogage tempo
					if ((n[1]=='*')||(n[1]=='C')) {pC->debug("*");}; // debogage compteurs
					if ((n[1]=='*')||(n[1]=='G')) {pG->debug("*");}; // debogage graphes
					if ((n[1]=='*')||(n[1]=='S')) {pS->debug("*");}; // debogage E/S série	
					if ((n[1]=='*')||(n[1]=='P')) {pPi->debug("*");}; // debogage E/S série	
					if (n[1]=='L'){debugTempsCycle();}// debogage temps cycle
				}
				else
				{
					pB->debug("*"); pT->debug("*"); pC->debug("*"); pG->debug("*"); pS->debug("*");pPi->debug("*");
					// debogage total
				}
			}
			else
			{
				pB->debug(n); // debogage selectif bascules avec le nom
				pT->debug(n); // debogage selectif tempo avec le nom
				pC->debug(n); // debogage selectif compteur avec le nom
				pG->debug(n); // debogage selectif graphes avec le nom
				pS->debug(n); // debogage selectif entrées sorties séries
				pPi->debug(n); // debogage selectif des PID
			}
		}
		else
		{
			pB->debug("*"); pT->debug("*"); pC->debug("*");pG->debug("*");pS->debug("*"); pPi->debug("*");
			// debogage total
		}
		SerialDebug->println("=*=*=*=");
	}
}

// ajoute une tempo au projet
void project::add(tempo *T)
{
	pT->add(T);
}

// ajoute un compteur au projet
void project::add(compt *C)
{
	pC->add(C);
}
// ajoute une bascule au projet
void project::add(bascule *B)
{
	pB->add(B);
}

// ajoute un graph au projet
void project::add(graph *G)   
{
	pG->add(G);
}

// ajoute un serialInOut au projet
void project::add(serialInOut *S)   
{
	pS->add(S);
}

// ajoute un pid au projet
void project::add(pid *P)   
{
	pPi->add(P);
}
// Le cour du systeme 
void project::fonctionnement()
{ 

	if (mode) // si mode run
	{
		preliminaire();
		action1();
		action2();
		evolution();
		action4();
		posterieur();
	}
	if (SerialDebugInitProject)
	{ 
		SerialDebugSto=SerialDebug; // on sauve les valeurs
		SerialDebugInitSto=SerialDebugInit;
		SerialDebug=SerialDebugProject; // redirection vers la sortie debug du projet
		SerialDebugInit=true;
		if (SerialDebug->available())
		{
			byte c =toupper(SerialDebug->read());
			
			switch (c) 
			{
			case 10 : executeCommande(); break;
			case 13 : executeCommande(); break;
			case '?': ligneCommande='?'; break;
			case '#': ligneCommande='#'; break;
				default : 
				{
					if ((c=='*')||((c>='A')&&(c<='Z'))||((c>='0')&&(c<='9'))){ligneCommande+=char(c);};
					break;
				}
			}
			
		}
		
		
		if (modeDebug) {debug(ligneDebug);}
		SerialDebug=SerialDebugSto; // on remet les valeurs initiales
		SerialDebugInit=SerialDebugInitSto;
	}
	unsigned long t=tempsAvant;
	tempsAvant=micros();
	tempsCycle=Delai_entre(t,tempsAvant); // calcul du temps de cycle
}

// pour traiter les commandes 
void project::executeCommande()
{
	if (ligneCommande.length()>0) // au moins un caractere de commande
	
	{
		if(ligneCommande[0]=='#')
		{
			switch (ligneCommande[1])
			{
			case 'R' : // mode run
				mode=true; 
				SerialDebug->println("#R");
				break;
			case 'S' : // mode stop
				mode=false; 
				SerialDebug->println("#S");
				break;
			case 'I' : // mode reinitialisation
				reInit();
				SerialDebug->println("#I");
				break; 
			case 'D' : // mode debug
				modeDebug=true;
				SerialDebug->println("#D");
				break;
			case 'N' :  // mode non debug
				modeDebug=false;
				SerialDebug->println("#N");
				break;	
			case 'L' : 
				SerialDebug->println("#L");
				SerialDebug->print(float(tempsCycle)/1000,3);SerialDebug->println(" ms");
				break;
			}
		}
		
		else
		{
			if (ligneCommande[0]=='?') // cas de la ligne de commande de debogage
			{
				ligneDebug=ligneCommande.substring(1); // On suprime le premier caractère
				if (ligneDebug=="") {ligneDebug="*";} // traitement du cas commande ="?" ==> tout debuger
				SerialDebug->print("?");SerialDebug->println(ligneDebug); // affiche de compte rendu
				debug(ligneDebug); // on envoi le debogage
			}
			
		}
		
	}
	ligneCommande="";
}
void project::debugTempsCycle()
{
	if (SerialDebugInit)
	{
		SerialDebug->print("Cycle #l : ");
		SerialDebug->print(float(tempsCycle)/1000,3);SerialDebug->println(" ms");
	}
}

// Retourne le dernier temps de cycle en microsecondes
long project::getCycle()
{
	return tempsCycle;
}

// Reinitialisation du projet
void project::reInit()
{
	pB->reInit(); // réinitialisation des objets de type bascule
	pT->reInit(); // réinitialisation des objets de type Tempo
	pG->reInit(); // réinitialisation des objets de type Graph
	pS->reInit(); // réinitialisation des objets de type SerialInOut
	pC->reInit(); // réinitialisation des objets de type Compteur
}

// evolution des états des objets du projet
void project::evolution()
{
	pB->evolution(); // evolution des objets de type bascule
	pG->evolution(); // evolution des objets de type graph
	pPi->evolution(); // evolution des objets de type PID
}
// Fontion preliminaire non négociapbles
void project::preliminaire()
{
	pS->preliminaire();
	pT->evolution(); // evolution des objets de type Tempo
	pC->evolution(); // evolution des objets de type compteurs
}

// fonctions postérieur non négociables
void project::posterieur()
{
	pS->posterieur();
}

// initialise le mode debogage
void project::debugMode(HardwareSerial *S , unsigned int speed)
{
	SerialDebugProject = S;
	S->begin(speed);
	SerialDebugInitProject = true;
}

// Les méthodes de l'objet bascule
// *******************************
// Le constructeur par defaut

// Les constructeurs "améliorés"


bascule::bascule(byte act,String n)
{
	typeBascule(act);
	n.toUpperCase();
	nom=n;
}

// La mise en blace du type de bascule
void bascule::typeBascule(byte act)
{
	etat = act;
}

// L'etat de la bascule
boolean bascule::isActif()
{
	return (boolean)(etat & etat_actif );
}

// préparation du basculement de la bascule
void bascule::setActif(boolean act)
{
	if (act) {
		etat |= etat_mise_a_1;
	}
	else {
		etat |= etat_mise_a_0;
	}
}

// Préparation du forcage de la bascule
// utilisé en mode debug pour une bascule
// et en mode normal pour la classe derivée etape

void bascule::setForcage(boolean act)
{
	if (act) {
		etat |= etat_forcage_1;
	}
	else {
		etat |= etat_forcage_0;
	}
}

// Evolution en tenant compte des priorités de mise à 0 et à 1
void bascule::evolution()
{  if (etat & etat_mise_a_1_prio)
	{evolutionUnPrioritaire();}
	else
	{evolutionZeroPrioritaire();}
}

// Evolution Mise à un prioritaire
void bascule::evolutionUnPrioritaire()
{
	byte etat_sto = etat;
	byte etat_config = etat & etat_bits_config;
	// Les forcages sont prépondérants sur les autres évolutions
	if ((etat_sto & etat_forcage_1))
	{ etat = etat_config | etat_actif; // on a traité !!
		return; // fin
	}
	if (etat_sto & etat_forcage_0)
	{ etat =  etat_config; // on a traité !!
		return; // fin
	}
	// Traitement des autres évolutions
	if  (~(etat_sto & etat_fige)) // si non figé (utilisé plutôt dans les classes dérivées (etape)
	{
		// Cas normal d'évolution par activation/desactivation
		if (etat_sto & etat_mise_a_1) // on traite la mise à 1 avant car elle est prépondérante
		{
			etat = etat_config | etat_actif;
			return; // fin
		}
		if (etat_sto & etat_mise_a_0)
		{
			etat = etat_config ;
			return; // fin
		}
		// pas d'evolution en vue, on conserve l'état actuel
	}
	else
	{
		//si figé, on n'evolue pas, on defige c'est tout;
		etat = etat_sto & (~etat_fige); // on defige c'est tout
	}
}

// Evolution Mise à zero prioritaire
void bascule::evolutionZeroPrioritaire()
{
	byte etat_sto = etat;
	byte etat_config = etat & etat_bits_config;
	// Cas de la mise à 0 prioritaire
	// Les forcages sont prépondérants sur les autres évolutions
	if (etat_sto & etat_forcage_0)
	{ etat = etat_config ; // traité !!
		return; // fin
	}
	if (etat_sto & etat_forcage_1)
	{ etat = etat_config | etat_actif; // traité
		return; // fin
	}
	// Traitement des autres évolutions
	if (~(etat_sto & etat_fige)) // si non figé
	{
		if  (etat_sto & etat_mise_a_0)
		{
			etat = etat_config;
			return;
		}
		if (etat_sto & etat_mise_a_1)
		{
			etat = etat_config | etat_actif;
			return;
		}
		// pas d'evolution en vue, on conserve l'état actuel
	}
	else
	{
		//si figé, on n'evolue pas, on defige c'est tout;
		etat = etat_sto & (~etat_fige); // on defige c'est tout
	}
}

void bascule::debug()
{
	if (SerialDebugInit)
	{
		SerialDebug->print(nom); SerialDebug->print(" : ");
		if (isActif()) {SerialDebug->println( "on ");}
		else {SerialDebug->println( "off");}
	}
}

void bascule::setName(String n)
{
	n.toUpperCase();
	nom=n;
}
// Les méthodes de l'objet etape
//******************************

// Les constructeurs dérivés
etape::etape(boolean initial)
{
	bascule(etat_mise_a_1_prio);
	typeEtape(initial);
	if (initial) {
		init(); // mise à 1 des étapes initiales
	}
	etat_front=0;
}

// Le taitement particulier des étapes initiales
void etape::reInit()
{
	etat &= etat_bits_config; // on ne garde que les bits de configuration

	if (etat & etat_initial) 
	{
		etat |=  etat_actif; // mise à 1 des étapes initiales
	}
	etat_front = 0;
}

// La surcharge de la methode evolution de l'objet bascule
void etape::evolution()
{
	boolean  etat_anterieur = isActif(); // stockage pour traitement des fronts

	bascule::evolutionUnPrioritaire(); // Les étapes sont à mise à un prioritaire
	// traitement des fronts d'étapes
	etat_front = 0;
	if ((! etat_anterieur) &&  (boolean)(etat & etat_actif))
	{
		etat_front = etat_fmt;
	}
	if ( etat_anterieur &&  (!(boolean)(etat & etat_actif)))
	{
		etat_front = etat_fdcd;
	}
}

// detection des fronts
boolean etape::fmt()
{
	return (boolean)(etat_front & etat_fmt);
}

boolean etape::fdcd()
{
	return (boolean)(etat_front & etat_fdcd);
}

// defini le type de l'etape (initial : true ; normale : false)
void etape::typeEtape(boolean t)
{ if (t)
	{
		typeBascule(etat_mise_a_1_prio | etat_initial | etat_actif);
	}
	else
	{
		typeBascule(etat_mise_a_1_prio);
	}
	etat_front = 0;
}

// fige l'étape
void etape::setFigeage()
{
	etat |= etat_fige;
}

// Les méthodes de l'objet pid
// *****************************


// Le constructeur en choisissant le type , le temps et le nom
pid::pid(double Kpo, double Kde, double Kin,String s) 
{
	setCoefParal(Kpo,Kde,Kin);
	s.toUpperCase();
	nom=s;
	reInit();
	mode=paralCoef;
}
// mets de nouveaux coef
void pid::setCoefParal(double Kpo, double Kde, double Kin)
{ 
	Kp=Kpo;
	Kd=Kde;
	Ki=Kin;
	mode=paralCoef;
}
// mets de nouveaux coef suivant les modes 
void pid::setCoeftimeParal(double Kpo, double Tde, double Tin)
{ 
	Kp=Kpo;
	Kd=Tde;
	if (Tin>0) {Ki=1/Tin;} else {Ki=1E20;}
	mode=paralTime;
}

void pid::setCoefmixte(double Kpo, double Kde, double Kin)
{ 
	Kp=Kpo;
	Kd=Kde*Kp;
	Ki=Kin*Kp;
	mode=mixteCoef;
}
// mets de nouveaux coef
void pid::setCoeftimemixte(double Kpo, double Tde, double Tin)
{ 
	Kp=Kpo;
	Kd=Tde*Kp;
	if (Tin>0) {Ki=Kp/Tin;} else {Ki=1E20;}
	mode=mixteTime;
}
// arrete ou met en marche le pid
void pid::on()
{
	etat=true;
	// redemarrage sans douleur
	sumValue=0; // on vide le coef integral
		
}
void pid::off()
{
	etat=false;
}

// renvoi l'état du module pid
boolean pid::state()
{
	return etat;
}
// est elle terminée ? renvoi l'état de la sortie
double pid::out()
{
	return sortie;
}

// forcage de la sortie en mode off_type
void pid::setOut(double s)
{
	if (!etat) {sortie=s;}
}

// force la désactivation (réinitialisation)
void pid::reInit()
{ 
	delaiScrutation=100000; // 100 ms
	etat=true; // en marche
	retour=0;
	sortie=0;
	epsilonSto=0;
	sumValue=0;
	dernierTraitement=tempsCourantBibliAutomMicros;
}

// impose des limites à la sortie
void pid::setOutputLimits(double max, double min) //calibre la sortie dans une plage donnée. 0-255 par defaut (pilotage d'une sortie anna 8 bits)
{
	if (min<max){double sto=max; max=min; min=sto;};
	if (max>min)
	{
		maxi=max;
		mini=min;
		if (sumValue>maxi) {sumValue=maxi;}
	}
}

// evolution du pid 
void pid::evolution()
{ 
	if (etat)
	{
		double t=Delai_entre(dernierTraitement,tempsCourantBibliAutomMicros);
		if (t>=delaiScrutation)
		{
			t=t/1000000; // conversion en seconde
			double epsilon=sortie-retour;
			sumValue=sumValue+t*epsilon; // integrale
			if (sumValue > maxi) {sumValue=maxi;};
			if (sumValue < mini) {sumValue=mini;};
			
			// calibration de l'integrale
			double derivValue=(epsilon-epsilonSto)/t;
			epsilonSto=epsilon;
			sortie=Kd*epsilon+Ki*sumValue+Kd*derivValue;
			if (sortie>maxi) {sortie=maxi;}; // calibration de la sortie
			if (sortie<mini) {sortie=mini;}; // calibration de la sortie
			dernierTraitement=tempsCourantBibliAutomMicros;	// actualisation	
		}
	}
	else
	{
		dernierTraitement=tempsCourantBibliAutomMicros;
	}
}



// le nom
void pid::setName(String n)
{
	n.toUpperCase();
	nom=n;
}

// debugging
void pid::debug()
{
	if (SerialDebugInit)
	{
		SerialDebug->print(nom); 
		SerialDebug->print(": PID ");
		if (state()) 
		{SerialDebug->print("<On>  ");}
		   else 
		{SerialDebug->print("<Off> ");}
	
		SerialDebug->print("Kp=");
		SerialDebug->print(Kp,2); // 2 digit derriere la virgule
		switch (mode)
		{ 
		case paralTime :
			SerialDebug->print(" ; Ti=");
			if (Ki>0) {SerialDebug->print(1/Ki,2);} // 2 digit derriere la virgule
			else {SerialDebug->print("infin ");}
			SerialDebug->print(" (s) ; Td=");
			SerialDebug->print(Kd,2); // 2 digit derriere la virgule
			
			break;
		case paralCoef :
			SerialDebug->print(" ; Ki=");
			SerialDebug->print(Ki,2); // 2 digit derriere la virgule
			SerialDebug->print(" (1/s) ; Kd=");
			SerialDebug->print(Kd,2); // 2 digit derriere la virgule
			
			break;
		case mixteTime :
			SerialDebug->print(" ; Ti=");
			if (Ki>0) {SerialDebug->print(1/Ki,2);} // 2 digit derriere la virgule
			else {SerialDebug->print("infin ");}
			SerialDebug->print(" (s) ; Td=");
			SerialDebug->print(Kd,2); // 2 digit derriere la virgule
			
			break;
		case mixteCoef :
			SerialDebug->print(" ; Ki=");
			if (Kp>0)
			{SerialDebug->print(Ki/Kp,2); // 2 digit derriere la virgule
				SerialDebug->print(" (1/s) ; Kd=");
				SerialDebug->print(Kd/Kp,2); // 2 digit derriere la virgule
			}
			else
			{SerialDebug->print("infin"); // 2 digit derriere la virgule
				SerialDebug->print(" (1/s) ; Kd=");
				SerialDebug->print("infin"); // 2 digit derriere la virgule
			}
			
			break;	
		}
		SerialDebug->print(" (s) => Eps=");
		SerialDebug->print(epsilonSto,1); // 1 digit derriere la virgule
		SerialDebug->print(" out=");
		SerialDebug->print(sortie,1); // 1 digit derriere la virgule 
		SerialDebug->println();
		
	}
}

//  change delaiScrutation
void pid::setDeltaT(unsigned long d) // d en ms;
{
	if (d>0)
	{ 
		delaiScrutation=d*1000; // converti en micosecondes
	}
}

// initialise la valeur courante de retour
void pid::setFeedBack(double v)
{
	retour=v;
}

// Les méthodes de l'objet tempo
// *****************************

void tempo::evolutionTon() // evolution pour tempo de type ton (tempo à l'enclenchement)
{   
	if (activationAsked) // on demande ou continue de demander l'activation
	{
		if (!actived) {tinit=tempsCourantBibliAutomMillis;} // réactualisation sur front montant de la commande
		actived=true; // officiellement activée
		// puis on demande si le delai de carence est ecoulé
		if (Delai_entre(tinit,tempsCourantBibliAutomMillis)>=delaisMs) {etat=tempoTon|etat_actif;} 
		activationAsked=false; // opération traitée
	}
	else
	{
		actived=false; // non activé
		etat= tempoTon; // on réinitialise le mot d'état donc la sortie
	}

}

void tempo::evolutionTof() // evolution de la tempo de type tof (tempo au declenchement)

{   
	if (activationAsked) // on demande ou continue de demander l'activation
	{
		actived=true; // officiellement activée
		etat=tempoTof|etat_actif; // sortie à 1	
		activeSto=false; // dans la première phase d'évolution	
		activationAsked=false; // opération traitée

	}
	else
	{
		if (actived) // réactualisation sur front descendant de la commande
		{
			tinit=tempsCourantBibliAutomMillis;
			activeSto=true; // pour la poursuite de l'évolution 
			actived=false; // officiellement commande désactivée
		} 
		// puis on demande si le delai de carence est ecoulé
		if (activeSto) {activeSto=(Delai_entre(tinit,tempsCourantBibliAutomMillis)<=delaisMs);};
		if (activeSto) 
		{etat=tempoTof | etat_actif;} // on active la sortie
		else 
		{etat=tempoTof;} // on desactive la sortie
	}

}

void tempo::evolutionTpulse() // evolution pour tempo de type tpulse (impulsion calibrée)
{   
	
	if (activationAsked) // on demande ou continue de demander l'activation
	{
		if (!actived) // front montant d'activation
		{
			tinit=tempsCourantBibliAutomMillis; // reinitialisation sur front montant
			actived=true; // officiellement activée
			activeSto=true; // dans la deuxieme phase d'évolution	
		}
		activationAsked=false; // opération traitée
	}
	else
	{   
		actived=false; // officiellemnt commande desactivée
	}
	// on traite le problème de delais de carence
	if (activeSto)
	{activeSto=(Delai_entre(tinit,tempsCourantBibliAutomMillis)<=delaisMs);}; // evolution d'activsto
	if (activeSto) 
	{etat=tempoTpulse | etat_actif;} // on active la sortie
	else 
	{etat=tempoTpulse;} // on desactive la sortie

}

// remets les indicateur au repos
void tempo::resetIndic()
{
	actived=false;
	activeSto=false;
	activationAsked=false;
	reinitAsked=false;
}
// Le constructeur par defaut ton 0 ms nom = ""
tempo::tempo()
{ 
	etat=etat_ton;
	delaisMs=0;
	resetIndic();
	nom="";
}

// Le constructeur en choisissant le type , le temps et le nom
tempo::tempo(byte t,unsigned long tpulse,String s) 
{
	switch (t)
	{
	case etat_tof : etat=etat_tof; break;
	case etat_tpulse : etat=etat_tpulse; break;
		default : etat=etat_ton; break;
	}
	resetIndic();
	delaisMs=tpulse;
	s.toUpperCase();
	nom=s;
}

// est elle terminée ? renvoi l'état de la sortie
boolean tempo::out()
{
	return  (etat & etat_actif);
}

// force la désactivation (réinitialisation)
void tempo::reInit()
{
	reinitAsked=true;
	
}

// demande activation de la tempo
void tempo::active()
{
	activationAsked=true;
}

// evolution de la tempo 
void tempo::evolution()
{ if (reinitAsked) // action prioritaire sur evolution normale
	{
		etat & (etat_ton|etat_tof|etat_tpulse);
		resetIndic();
	}	
	else
	{
		byte etatsto=etat & (tempoTon|tempoTof|tempoTpulse); // on recupére le type de tempo
		switch (etatsto) // suivant le type de tempo
		{ 
		case tempoTpulse :
			evolutionTpulse();
			break;
		case tempoTof :
			evolutionTof();
			break;
		case tempoTon :
			evolutionTon();
			break;
		}
	}
}



// le nom
void tempo::setName(String n)
{
	n.toUpperCase();
	nom=n;
}

// debugging
void tempo::debug()
{
	if (SerialDebugInit)
	{
		SerialDebug->print(nom); SerialDebug->print(" : <");
		byte typeTempo=etat & (tempoTon|tempoTof|tempoTpulse);
		
		switch (typeTempo) // suivant le type de tempo
		{ case etat_tpulse :
			SerialDebug->print("Tpu> : ");
			break;
		case etat_tof :
			SerialDebug->print("Tof> : ");
			break;
		case etat_ton :
			SerialDebug->print("Ton> : ");
			break;
		}
		if (actived) // etat de l'entrée 
		{SerialDebug->print( "act ; ");}
		else
		{SerialDebug->print( "/act; ");}
		
		if (etat&etat_actif) // etat de la sortie
		{SerialDebug->print( "on  ;");}
		else
		{SerialDebug->print( "off ;");}
		
		switch (typeTempo) // suivant le type de tempo
		{ case etat_tpulse : // tempo type pulse
			if (activeSto)
			{
				SerialDebug->print(delaisMs-Delai_entre(tinit,tempsCourantBibliAutomMillis));
				SerialDebug->println(" ms remain");
				break;
			}
			SerialDebug->println("0 ms remain");  
			break;
			
		case etat_ton : // tempo type on
			if (actived && !boolean(etat & etat_actif))
			{
				SerialDebug->print(delaisMs-Delai_entre(tinit,tempsCourantBibliAutomMillis));
				SerialDebug->println(" ms remain");
				break;
			}
			SerialDebug->println("0 ms remain");  
			break;
		case etat_tof :
			if (!actived && boolean(etat & etat_actif))
			{
				SerialDebug->print(delaisMs-Delai_entre(tinit,tempsCourantBibliAutomMillis));
				SerialDebug->println(" ms remain");
				break;
			}
			SerialDebug->println("0 ms remain");  
			break;
		}
	}
}

// Les méthodes de l'objet compt
// *****************************

void compt::evolution() // evolution pour compt de type ton (compt à l'enclenchement)
{   
	if (modifPreselAsked)
	{
		presel=newPresel;
		modifPreselAsked=false;
	}
	if (reinitAsked) // reinitialisation à la valeur presel
	{
		resetIndic();
		valeur=presel;
	}
	else
	
	{ if (resetAsked) // remise à 0 prioritaire
		{
			resetIndic();
		}		
		else
		{
			// traitement normal !!
			if (incrementAsked || decrementAsked)
			{
				if (incrementAsked) // on demande ou continue de demander l'activation
				{
					if (!activedinc) // front montant d'incrementation
					{
						valeur++; // incrementation
						activedinc=true; // officiellement traitée
					}
					incrementAsked=false; // opération traitée
				}
				else
				{
					activedinc=false;
				}
				if (decrementAsked) // on demande ou continue de demander la decrementation
				{
					if (!activeddec) // front montant de decrementation
					{
						valeur--; // incrementation
						activeddec=true; // officiellement traitée
					}
					decrementAsked=false; // opération traitée
				}
				else
				{
					activeddec=false;
				}
			}
			else
			{
				activedinc=false;
				activeddec=false;
			}
		}
	}
	etat=etat & (~(etat_actif|etat_sup|etat_inf)); // effacer les indicateurs
	// puis on remet les bons
	if (valeur==presel) {etat=etat | etat_actif; return;}; 
	if (valeur>presel) {etat=etat | etat_sup; return;};
	etat=etat | etat_inf; 
	
}
void compt::resetIndic()
{
	incrementAsked=false;
	decrementAsked=false;
	reinitAsked=false;
	resetAsked=false;
	activedinc=false;
	activeddec=false;
	etat=0x0;
	valeur=0;
	modifPreselAsked=false;
	
}

// Le constructeur en choisissant la preselection et le nom
compt::compt(long pre,String s) 
{
	presel=pre;
	resetIndic();
	s.toUpperCase();
	nom=s;
}

// renvoi la valeur courante du compteur
long compt::val()
{
	return valeur;
}

// modifie la valeur de preselection

void compt::setPresel(long p)
{
	modifPreselAsked=true;
	newPresel=p;
}

// est il à la preselection ?
boolean compt::isActif()
{
	return  (etat & etat_actif);
}

boolean compt::isEqual()
{
	return  (etat & etat_actif);
}

// est il < la preselection ?
boolean compt::isLower()
{
	return  (etat & etat_inf);
}

// est il > la preselection ?
boolean compt::isUpper()
{
	return  (etat & etat_sup);
}

// valeur courante =0 ?
boolean compt::isZero()
{
	return !(boolean(valeur));
	
}
// force la mise à 0
void compt::reInit()
{
	resetAsked=true;
}
void compt::reSet()
{
	resetAsked=true;
}

// force la mise à preselection
void compt::preSet()
{
	reinitAsked=true;
}

// demande l'incrementation du compteur
void compt::comptUp()
{
	incrementAsked=true;
}

// demande la decrementation du compteur
void compt::comptDown()
{
	decrementAsked=true;
}


// le nom
void compt::setName(String n)
{
	n.toUpperCase();
	nom=n;
}

// debugging
void compt::debug()
{
	if (SerialDebugInit)
	{
		SerialDebug->print(nom); SerialDebug->print(" : compt < pre : ");
		SerialDebug->print(presel);
		SerialDebug->print(" > , val : ");
		SerialDebug->print(valeur);
		if (etat & etat_actif) {SerialDebug->println( " =="); return;}
		if (etat & etat_sup) {SerialDebug->println( " >>");return;}
		SerialDebug->println( " <<"); return;
		
	}
}

// Les méthodes de l'objet graph
// *****************************


// le module de d=creation
void graph::creation(int n)
{
	nb = n;
	et = new etape [nb]; // Creation d'une zone mémoire de taille nb*sizeof(etape) attention c'est un tableau donc et pointe sur la premiere case
	for (unsigned i = 0; i < nb; i++)
	{
		et[i].typeEtape(false); // et[i] repesente l'étape N° i c'est en fait un pointeur
	}
}


// Le constructeur amélioré N°4
graph::graph(int n , int i,String s)
{
	creation (n);
	init(i); // mise en place de l'étape initiale
	s.toUpperCase();
	setName(s);
	
}
// Création de l'étape initiale
void graph::init( int n)
{
	if ((n>=0) & (n < nb)) {
		et[n].typeEtape(true);
	}
}

// Remise du graph à son état de création
void graph::reInit()
{
	for (int i = 0; i < nb; i++) {
		et[i].reInit();
	}
}

// evolution du graph
void graph::evolution()
{
	for (unsigned i = 0; i < nb; i++)
	{
		et[i].evolution();
	}
}

// rechercher une étape particulière du graphe (un pointeur sur une étape)
etape* graph::contenu(unsigned int n)
{
	if (n < nb) {
		return (&et[n]); // renvoi l'adresse de l'etape
	}
	else {
		return NULL;
	}
}

// fige le graphe à son état actuel
void graph::fige()
{
	for (unsigned int i = 0; i < nb; i++)
	{
		et[i].setFigeage();
	}
}
// mouchard pour debogage
void graph::debug(String n)
{
	if (SerialDebugInit)
	{ if ((n==nom) || (n="*")|| (nom=""))
		{
			SerialDebug->print(nom); SerialDebug->print(" : ");
			for (int i = 0; i < nb; i++)
			{
				if (et[i].isActif())
				{
					SerialDebug->print("X");
					SerialDebug->print(i);
					SerialDebug->print(" ");
				}

			}
			SerialDebug->println();
		}
	}
}

// donne un nom au graphe

void graph::setName(String n)

{
	n.toUpperCase();
	nom = n;
}
// renvoi le nombre d'étapes
int graph::getNb() {return nb;};

// desactive le graph (le force à 0 pour le tour de calcul suivant)
void graph::unActiv()
{
	for (int i=0;i<nb;i++) {et[i].setForcage(false); }
}

// force à 1 une etape donnée
void graph::forceTo1(int i)
{
	if ((i>0)&(i<nb)){et[i].setForcage(true);}
}
// Les méthodes de l'objet pilPid
// ******************************


pilePid::pilePid(pid *p) // g est un pointeur sur un objet de type graph
{
	next=NULL;
	contents=p;
}

void pilePid::evolution()
{
	if (contents!=NULL) {contents->evolution();} // on pilote l'évolution du graph pointé
	if (!(next==NULL))
	{next->evolution();}; // on monte dans la liste chainée
}

void pilePid::debug(String n)
{
	if (contents!=NULL)
	{
		if ((n=="")||(n=="*")||(n==contents->nom)) // soit tous les graph soit le graph ciblé (nom n)
		{
			contents->debug();
		}
	}
	if (next!=NULL)
	{
		next->debug(n);// on monte dans la liste chainée
	} 
}

void pilePid::add(pid *p)
{
	if (contents==NULL) {contents=p; return;}; // on rempli une case vide et on a fini
	if (next!=NULL) {next->add(p); return;}; // on va voir la suite de la chaine et on a fini
	next=new pilePid(p); // on est manifestement en bout de chaine donc on ajoute un petale
}

void pilePid::reInit()
{
	if (!(next==NULL))
	{next->reInit();}; // on monte dans la liste chainée
	if (contents!=NULL) {contents->reInit();} // on pilote la réinitialisation du graph pointé
}
// Les méthodes de l'objet pilGraph
// ********************************


pileGraph::pileGraph(graph *g) // g est un pointeur sur un objet de type graph
{
	next=NULL;
	contents=g;
}

void pileGraph::evolution()
{
	if (contents!=NULL) {contents->evolution();} // on pilote l'évolution du graph pointé
	if (!(next==NULL))
	{next->evolution();}; // on monte dans la liste chainée
}

void pileGraph::debug(String n)
{
	if (contents!=NULL)
	{
		if ((n=="")||(n=="*")||(n==contents->nom)) // soit tous les graph soit le graph ciblé (nom n)
		{
			contents->debug(n);
		}
	}
	if (next!=NULL)
	{
		next->debug(n);// on monte dans la liste chainée
	} 
}

void pileGraph::add(graph *g)
{
	if (contents==NULL) {contents=g; return;}; // on rempli une case vide et on a fini
	if (next!=NULL) {next->add(g); return;}; // on va voir la suite de la chaine et on a fini
	next=new pileGraph(g); // on est manifestement en bout de chaine donc on ajoute un petale
}

void pileGraph::reInit()
{
	if (!(next==NULL))
	{next->reInit();}; // on monte dans la liste chainée
	if (contents!=NULL) {contents->reInit();} // on pilote la réinitialisation du graph pointé
}
// Les méthodes de l'objet pilTempo
// ********************************

pileTempo::pileTempo(tempo *t) // g est un pointeur sur un objet de type graph
{
	next=NULL;
	contents=t;
}


void pileTempo::debug(String n)
{
	if (contents!=NULL)
	{
		if ((n=="*")||(n==contents->nom)) // soit tous les graph soiot le graph ciblé (nom n)
		{
			contents->debug();
		}
	}
	if (next!=NULL)
	{
		next->debug(n);// on monte dans la liste chainée
	} 
}

void pileTempo::add(tempo *t)
{
	if (contents==NULL) {contents=t; return;}; // on rempli une case vide et on a fini
	if (next!=NULL) {next->add(t); return;}; // on va voir la suite de la chaine et on a fini
	next=new pileTempo(t); // on est manifestement en bout de chaine donc on ajoute un petale
}

void pileTempo::reInit()
{
	if (!(next==NULL))
	{next->reInit();}; // on monte dans la liste chainée
	if (contents!=NULL) {contents->reInit();} // on pilote la réinitialisation de la tempo pointée
}

void pileTempo::evolution()
{
	if (contents!=NULL) {contents->evolution();} // on pilote l'évolution du graph pointé
	if (!(next==NULL))
	{next->evolution();}; // on monte dans la liste chainée
	
}

// Les méthodes de l'objet pilcompt
// ********************************

pileCompt::pileCompt(compt *c) // g est un pointeur sur un objet de type graph
{
	next=NULL;
	contents=c;
}


void pileCompt::debug(String n)
{
	if (contents!=NULL)
	{
		if ((n=="*")||(n==contents->nom)) // soit tous les compteurs soit le compteur ciblé (nom n)
		{
			contents->debug();
		}
	}
	if (next!=NULL)
	{
		next->debug(n);// on monte dans la liste chainée
	} 
}

void pileCompt::add(compt *c)
{
	if (contents==NULL) {contents=c; return;}; // on rempli une case vide et on a fini
	if (next!=NULL) {next->add(c); return;}; // on va voir la suite de la chaine et on a fini
	next=new pileCompt(c); // on est manifestement en bout de chaine donc on ajoute un petale
}

void pileCompt::reInit()
{
	if (!(next==NULL))
	
	{next->reInit();}; // on monte dans la liste chainée
	if (contents!=NULL) {contents->reInit();} // on pilote la réinitialisation de la compt pointée
}

void pileCompt::evolution()
{
	if (contents!=NULL) {contents->evolution();} // on pilote l'évolution du compteur pointé
	if (!(next==NULL))
	{next->evolution();}; // on monte dans la liste chainée
	
}

// Les méthodes de l'objet pilBascule
// **********************************

pileBascule::pileBascule(bascule *b) // g est un pointeur sur un objet de type graph
{
	next=NULL;
	contents=b;
}

void pileBascule::evolution()
{
	if (contents!=NULL) {contents->evolution();} // on pilote l'évolution de la bascule pointé
	if (!(next==NULL))
	{next->evolution();}; // on monte dans la liste chainée
	
}

void pileBascule::debug(String n)
{
	if (contents!=NULL)
	{
		if ((n=="*")||(n==contents->nom)) // soit tous les graph soit le graph ciblé (nom n)
		{
			contents->debug();
		}
	}
	if (next!=NULL)
	{
		next->debug(n);// on monte dans la liste chainée
	} 
}

void pileBascule::add(bascule *b)
{
	if (contents==NULL) {contents=b; return;}; // on rempli une case vide et on a fini
	if (next!=NULL) {next->add(b); return;}; // on va voir la suite de la chaine et on a fini
	next=new pileBascule(b); // on est manifestement en bout de chaine donc on ajoute un petale
}

void pileBascule::reInit()
{
	if (!(next==NULL))
	{next->reInit();}; // on monte dans la liste chainée
	if (contents!=NULL) {contents->reInit();} // on pilote la réinitialisation de la bascule (reset)
}
// Les méthodes de l'objet pilSerialInOut
// *************************************


pileSerialInOut::pileSerialInOut(serialInOut *s) // s est un pointeur sur un objet de type serialInOut
{
	next=NULL;
	contents=s; 
}

void pileSerialInOut::preliminaire()
{
	if (!(next==NULL))
	{next->preliminaire();}; // on monte dans la liste chainée
	if (contents!=NULL) {contents->preliminaire();} // on pilote l'évolution du serialInOut pointé
}
void pileSerialInOut::posterieur()
{
	if (!(next==NULL))
	{next->posterieur();}; // on monte dans la liste chainée
	if (contents!=NULL) {contents->posterieur();} // le traitement posterieur de la serialInOut pointé
}

void pileSerialInOut::add(serialInOut *s=NULL)
{
	if (contents==NULL) {contents=s; return;}; // on rempli une case vide et on a fini
	if (next!=NULL) {next->add(s); return;}; // on va voir la suite de la chaine et on a fini
	next=new pileSerialInOut(s); // on est manifestement en bout de chaine donc on ajoute un petale
}

void pileSerialInOut::reInit()
{
	if (!(next==NULL))
	{next->reInit();}; // on monte dans la liste chainée
	if (contents!=NULL) {contents->reInit();} // on pilote la réinitialisation de serialInOut (vidange des buffers)
}

void pileSerialInOut::debug(String n)
{
	if (contents!=NULL)
	{
		if ((n=="*")||(n==contents->nom)) 
		// soit toutes les sorties série soit la sortiesérie ciblé (nom n)
		{
			contents->debug();
		}
	}
	if (next!=NULL)
	{
		next->debug(n);// on monte dans la liste chainée
	} 
}
// Les méthodes de l'objet serialInOut
// ***********************************

// Les constructeurs

serialInOut::serialInOut(HardwareSerial *S , unsigned int s,String n)
{
	SerialConnected = S;
	SerialConnected->begin(speed);
	SerialConnectedInit = true;
	n.toUpperCase();
	nom=n;
	endBufferIn=0;
	endBufferOut=0;
	nbByteSend=0;
	speed=s;
}

// baptise le périphérique 
void serialInOut::setName(String n)
{
	n.toUpperCase();
	nom=n;
}
// procedure préliminaire qui sera traitée automatiquement
// dans la pileSerialInOut implanté dans l'objet project
// cette fonction récupère les bytes venant du buffer 
// d'entrée sortie du sérialisateur matériel si présents
void serialInOut::preliminaire()
{ int c;
	if (SerialConnectedInit)
	{ 
		while ((c=SerialConnected->read())>0) 
		{
			put(byte(c));
		}
	}
	nbByteSend=0;
}

// procedure postérieure qui sera traitée automatiquement
// dans la pileSerialInOut implanté dans l'objet project
// cette fonction envoi sur le buffer assynchrone du sérialisateur matériel 
// si il y en a dans le buffer de sortie

void serialInOut::posterieur()
{ int c;
	if (SerialConnectedInit)
	{ 
		while ((c=get())>0) 
		{
			SerialConnected->write(byte(c));
			nbByteSend++;
		}
	}
}


// decale le buffer de sortie de n crans vers la droite
void serialInOut::decaleDroite(int n)
{ 
	if (n>0)
	{
		endBufferOut=endBufferOut+n;
		if (endBufferOut>=(maxBuffer)) {endBufferOut=maxBuffer-1;};
		for (int i=endBufferOut; i>(n-1);i--) {bufferOut[i]=bufferOut[i-n];};
		for (int i=0; i<n;i++) {bufferOut[i]=0;};
	}
}

// renvoi le nombre de byte disponibles dans le buffer d'entrée
int serialInOut::available()
{ return endBufferIn;}

// renvoi le caractère disponible dans le buffer d'entrée -1 si aucuns
int serialInOut::get()
{
	if (endBufferIn>0)
	{
		endBufferIn--;
		return bufferIn[endBufferIn];
	}
	return (-1); // il n'y a rien dans le bufferIn
}

// mets un caractère dans le buffer de sortie
void serialInOut::put(byte b)
{
	decaleDroite();
	bufferOut[0]=b;
}

void serialInOut::emmet(numType &d, byte n)
{
	for (byte i=0; i<n; i++){put (d.t[i]);};
}
// mets un entier de type int(2 octets) dans le buffer
void serialInOut::put(int n)
{ 
	numType data;
	data.i=n;
	emmet(data,sizeof(int));
	
}

// mets un entier de type long(4 octets) dans le buffer
void serialInOut::put(long n)
{ 
	numType data;
	data.l=n;
	emmet(data,sizeof(long));
	
}
// mets un nombre de type reel (float) (4 octets) dans le buffer
void serialInOut::put(float f)
{ 
	numType data;
	data.f=f;
	emmet(data,sizeof(float));
	
}
// mets un nombre de type double (float) (4 octets) dans le buffer
void serialInOut::put(double d)
{ 
	numType data;
	data.d=d;
	emmet(data,sizeof(double));
	
}
// mets une chaine de caractère azt dans le buffer de sortie
void serialInOut::put(String s)
{
	for (int i=0;i<s.length();i++) {put(byte(s[i]));}
	put(0); // fin de chaine azt
}

//récupère une chaine azt du buffer d'entrée
void serialInOut::getString(String &s)
{
	int c;
	s="";
	while ((c=get())>0)
	{s=s+char(c);}
}

boolean serialInOut::recoit(numType &data,byte n)
{
	if ((n>0) &(available()>= n))
	{
		for (int i=0; i<n; i++) {data.t[i]=byte(get());};
		return true;
	}
	else
	{return false;}
	
}
// renvoi un entier (2 octets du buffer)
int serialInOut::getInt()
{
	numType data;
	if (recoit(data,sizeof(int))) {return data.i;} else {return 0;};
	
}

// renvoi un entier long (4 octets du buffer)
long serialInOut::getLong()
{
	numType data;
	if (recoit(data,sizeof(long))) {return data.l;} else {return 0;};
	
}

// renvoi un flotant (4 octets du buffer)
float serialInOut::getFloat()
{
	numType data;
	if (recoit(data,sizeof(float))) {return data.f;} else {return 0;};
	
}
// renvoi un flotant double (8 octets du buffer)
double serialInOut::getDouble()
{
	numType data;
	if (recoit(data,sizeof(double))) {return data.d;} else {return 0;};
	
}


// réinitialise les buffers
void serialInOut::reInit()
{
	endBufferOut=0;
	endBufferIn=0;
	nbByteSend=0;
}
// renvoi les informations sur la sortie série
void serialInOut::debug()
{
	if (SerialDebugInit)
	{
		SerialDebug->print(nom); SerialDebug->print(" : <");
		SerialDebug->print(speed);
		SerialDebug->print(" bds> ");
		SerialDebug->print(endBufferIn);
		SerialDebug->print(" in ; ");
		SerialDebug->print(nbByteSend);
		SerialDebug->println(" out");
	}
}




