


// et un petit exemple pour comprendre comment tout ce petit monde fonctionne
// ===========================================================================
// inclusion de la librairie
#include <Autom.h>
/*
La structure ci dessous permet de décrire un système de graphes
par exemple pour un automatisme à quatre graphes qui ne fait rien de bien
si ce n'est mettre en application le noyaux automate

========================
Graphe d'arret d'urgence

gau.et[1]
  v
  |
 +=+
 ║0║-- F/Gmode/X10;F/Gauto/X20;
 +++
  |
  + /au
  |
 +++
 |1|
 +++
  |
  + au
  |
  v
  gau.et[0]
 
======================== 
Graphe mode de marche
  
 +==+
 ║10║
 ++++                     X12 :gmanu.et[2]  X13 : gmanu.et[3]
  |                       v                 v
  + gau.et[1].isActif()   |                 |
  +-----------------------+-----------------+ 
  |
 ++++
 |11|- F/gauto/X20; voyant si (bitS6. cond_init)
 ++++
  |
  +--------------------+
  + auto. Cond_init    + manu
 ++++                 ++++
 |12| Vauto           |13| Vmanu;Vcinit si Cond_init
 ++++                 ++++
  + /auto              +/manu
  |                    |
  v                    v
  X11 :gmanu.et[1]     X11 : gmanu.et[1]
  
 
 ==========================
 Graphe mode automatique
 
  X23 : gauto.et[3]
  v
  |
 +==+
 ║20║
 ++++
  |
  + gmode.et[2].dcy^ 
  |
 ++++
 |21|- si X1^ armer T4  => if (gauto.et[1].isActif()) {T4.active()}
 ++++
  |
  + T4/gauto.et[1] / 10 s  => (T4.isActif())
  |
 ++++
 |22|- A1 si ana1>16 ; A2 si ana1>256
 ++++
  |
  + ana1>512 
  |
 ++++
 |23|
 ++++
  |
  + =1
  |
  v
  X20 : gauto.et[0]
 
 ==========================
 Graphe pour battre la seconde
 
  X31 : gsec.et[1]
  v
  |
 +==+
 ║30║ s6=0 armer tempo2  => if (gsec.et[0].isActif()) {T2.active()}
 ++++
  |
  + T2/gsec.et[0]/0.5 s => (T2.isActif())
  |
 ++++
 |31|- s6=1;armer tempo3  => if (gsec.et[1].fmt()) {T3.active()}
 ++++
  |
  + T3/gsecet[1] / 0.5 s  => (T3.isActif())
  |
  v
  X30 : gsec.et[0]
 */

// un objet appli dans lequel on va installer le système de graphes, tempos ....
project appli=project();

// Déclaration des graphes
graph gau=graph(2,0,"gau"); // un graphe d'arret d'urgence à 2 étapes avec 1 étape initiale
graph gmode=graph(4,0,"gmode"); // un graphe de mode de marche à 4 étapes avec 1 étape initiale
graph gauto=graph(4,0,"gauto"); // un graphe de marche auto à 4 étapes avec une étape initale
graph gsec=graph(2,0,"gsec"); // un graphe pour battre la seconde à 2 étapes avec une étape initale

// déclaration des tempos
tempo T2=tempo(tempoTon,500,"T2"); // tempo de 500 ms soit 0.5 s
tempo T3=tempo(tempoTon,500,"T3"); // tempo de 500 ms soit 0.5 s 
tempo T4=tempo(tempoTon,100,"T4"); // tempo de 10000 ms = 10s

// Les entrés
#define pinAu 2 // le bouton d'arrêt d'urgence branché sur l'entrée N°2
boolean aur =false; // bit associé au boutn d'arrêt d'urgence
#define pinAuto 3 // le bouton auto-manu sur l'entrée N°3
boolean automatique= false;
boolean manuel=!automatique;


#define pindcy 4 // le bouton dcy
boolean dcy = false;
boolean dcysto = false; // la valeur de stockage pour effectuer le front
boolean dcyfmt = false; // le front montant

#define pinvinit 5 // un voyant conditions initiales
#define pinvauto  6 // voyant auto
#define pinvmanu 7 // voyant manu
#define pinbargraph0 A1 // une ebauche de bargraph
#define pinbargraph1 A2
#define pinana A0 // l'entrée analogique

int ana1 = 0; // la valeur analogique entrée


// Les valeurs intermédiaires utiles
boolean cond_init=true; // les conditions initiales à calculer dans le préliminaire;
boolean bitS6 = false;


// Les procedures à écrire pour décrire le fonctionnement de l'application 
// ***********************************************************************

// Les phases de fonctionnement du système numérique associé
void preliminaire_appli() // calculs preliminaires ex fronts prise de valeurs digitales, analogiques ...
{
  aur=digitalRead(pinAu); // prise de l'état du bouton arrêt urgence
  automatique=digitalRead(pinAuto); // prise de l'etat du bouton auto/manu
  manuel=!automatique; // bouton à une galette avec forcement exclusion des deux états simultanés 
  cond_init=true; // à calculer
  
  dcy=digitalRead(pindcy); // traitement du bouton dcy
  dcyfmt=dcy & !dcysto;
  dcysto=dcy;
  
  ana1=analogRead(pinana); // l'entrée analogique
   
}


void calculEvolution_appli() // préparation de l'évolution des graphes en suivant les transitions
{
  // preparaton evolution graphe gsec
  // ********************************
/*
  X31 : gsec.et[1]
  v
  |
 +==+
 ║30║ s6=0 ; armer tempo2  => if (gsec.et[0].isActif()) {T2.active()}
 ++++
  |
  + T2/gsec.et[0]/0.5 s => (T2.isActif())
  |
 ++++
 |31|- s6=1 ; armer tempo3  => if (gsec.et[1].isActif()) {T3.active()}
 ++++
  |
  + T3/gsecet[1] / 0.5 s  => (T3.isActif())
  |
  v
  X30 : gsec.et[0]
 */
  // traitement transition etape 0 etape 1
  if (T2.out() & gsec.et[0].isActif()) 
  {
    gsec.et[0].setActif(false); // desactive l'etape antérieur
    gsec.et[1].setActif(true); // active l'etape postérieur
  }
  // traitement transition etape 1 et etape 0

  if (T3.out()& gsec.et[1].isActif()) 
  {
    gsec.et[1].setActif(false); // desactive l'etape antérieur
    gsec.et[0].setActif(true); // active l'etape postérieur
  }


  // préparation evolution graphe gau
  // ********************************
/*

gau.et[1]
  v
  |
 +=+
 ║0║-- F/Gmode/X10;F/Gauto/X20;
 +++
  |
  + /au
  |
 +++
 |1|
 +++
  |
  + au
  |
  v
  gau.et[0]
  
 */
  // traitement transition etape 0 etape 1
  if (!aur & gau.et[0].isActif()) 
  {
    gau.et[0].setActif(false); // desactive l'etape antérieur
    gau.et[1].setActif(true); // active l'etape postérieur
  }
  // traitement transition etape 1 et etape 0

  if (aur & gau.et[1].isActif()) 
  {
    gau.et[1].setActif(false); // desactive l'etape antérieur
    gau.et[0].setActif(true); // active l'etape postérieur
  }

  // préparation evolution graphe modes de marche
  // ********************************************
/*
======================== 
Graphe mode de marche
  
 +==+
 ║10║
 ++++                     X12 :gmanu.et[2]  X13 : gmanu.et[3]
  |                       v                 v
  + gau.et[1].isActif()   |                 |
  +-----------------------+-----------------+ 
  |
 ++++
 |11|- F/gauto/X20; voyant si (bitS6. cond_init)
 ++++
  |
  +--------------------+
  + auto. Cond_init    + manu
 ++++                 ++++
 |12| Vauto           |13| Vmanu;Vcinit si Cond_init
 ++++                 ++++
  + /auto              +/manu
  |                    |
  v                    v
  X11 :gmanu.et[1]     X11 : gmanu.et[1]
 */ 
  // traitement transition etape 0 etape 1
  if (gmode.et[0].isActif() && gau.et[1].isActif()) // && car il sagit d'évalution booleennes
  {
    gmode.et[0].setActif(false); // desactive l'etape antérieur
    gmode.et[1].setActif(true); // active l'etape postérieur
  }
  // traitement transition etape 2 etape 1
  if (gmode.et[2].isActif() && (!automatique))
  {
    gmode.et[2].setActif(false);
    gmode.et[1].setActif(true);
  }
  // traitement transition etape 3 etape 1
  if (gmode.et[3].isActif() && (! manuel))
  {
    gmode.et[3].setActif(false);
    gmode.et[1].setActif(true);
  }
  // traitement transition etape 1 etape 3
  if (gmode.et[1].isActif() && manuel )
  {
    gmode.et[1].setActif(false);
    gmode.et[3].setActif(true);
  }
  // traitement transition etape 1 etape 2
  if (gmode.et[1].isActif() && automatique  && cond_init)
  {
    gmode.et[1].setActif(false);
    gmode.et[2].setActif(true);
  }

  // préparation evolution graphe auto
  // *********************************
/*
X23 : gauto.et[3]
  v
  |
 +==+
 ║20║
 ++++
  |
  + gmode.et[2].dcy^ 
  |
 ++++
 |21|- si X1 armer T4  => if (gauto.et[1].isActive()) {T4.active()}
 ++++
  |
  + T4/gauto.et[1] / 10 s  => (T4.isActif())
  |
 ++++
 |22|- A1 si ana1>16 ; A2 si ana1>256
 ++++
  |
  + ana1>512 
  |
 ++++
 |23|
 ++++
  |
  + =1
  |
  v
  X20 : gauto.et[0]
*/ 
  // traitement transition etape 0 etape 1
  if (gauto.et[0].isActif() & dcyfmt & gmode.et[2].isActif())
  {
    gauto.et[0].setActif(false); // desactive l'etape antérieur
    gauto.et[1].setActif(true); // active l'etape postérieur
  }
  // traitement transition etape 1 etape 2
  if (gauto.et[1].isActif() & T4.out())
  {
    gauto.et[1].setActif(false);
    gauto.et[2].setActif(true);
  }
  // traitement transition etape 2 etape 3
  if (gauto.et[2].isActif() & (ana1>500))
  {
    gauto.et[2].setActif(false);
    gauto.et[3].setActif(true);
  }
  // traitement transition etape 3 etape 0
  if (gauto.et[3].isActif() ) // transition =1
  {
    gauto.et[3].setActif(false);
    gauto.et[0].setActif(true);
  }
}

void posterieur_appli() // typiquement affectation des sorties
{
  // Premièrement Traitement des figeages et forcages;
 
  if (gau.et[0].isActif()) {gmode.reInit();};
 
  if (gau.et[0].isActif()| gmode.et[1].isActif()) {gauto.reInit();}
  
  // Deuxiemement activer les tempos
  if (gsec.et[0].isActif()) {T2.active();};
  if (gsec.et[1].isActif()) {T3.active();};
  if (gauto.et[1].isActif()) {T4.active();};
  // Troisièmement Traitement des sorties
  // Les bits internes
  bitS6=gsec.et[1].isActif(); // bit qui bat la seconde
  // Les sortie tout ou rien externes
  digitalWrite(pinbargraph0,gauto.et[2].isActif() & (ana1>16));
  digitalWrite(pinbargraph1,gauto.et[2].isActif() & (ana1>250)); 
  digitalWrite(pinvinit,(gmode.et[1].isActif() & bitS6)|(gmode.et[3].isActif() & cond_init));
  digitalWrite(pinvmanu,gmode.et[3].isActif());
  digitalWrite(pinvauto,gmode.et[2].isActif());
}



void setup() // initialisation E/S séries ....
{
  pinMode(pinAu,INPUT_PULLUP);
  pinMode(pinAuto,INPUT_PULLUP);
  pinMode(pindcy,INPUT_PULLUP);
  pinMode(pinbargraph0,OUTPUT);
  pinMode(pinbargraph1,OUTPUT);
  pinMode(pinvinit,OUTPUT);
  pinMode(pinvauto,OUTPUT);
  pinMode(pinvmanu,OUTPUT);  
 
  // Mise en place du mode de debogage sur la sortie serie Serial
  appli.debugMode(&Serial,9600); // mise en place du mode de debogage
  
  // initialisation de l'application
  // on installe les entités dans l'application
  appli.add(&gau); 
  appli.add(&gmode);
  appli.add(&gauto);
  appli.add(&gsec);
  appli.add(&T2);
  appli.add(&T3);
  appli.add(&T4);

  // on affecte les fonctions
  appli.setPreliminaire(preliminaire_appli);
  appli.setCalculEvolution(calculEvolution_appli);
  appli.setPosterieur(posterieur_appli);
  
  
}


void loop() // Le chef d'orchestre (le vrai !)
{
  // mise à jour obligatoire des variables globales tous les tours de calcul
  initLoop(); 
	
  // tout le reste est dans appli !!
  appli.fonctionnement();
 
}


